#property copyright "Scriptong"
#property link "scriptong@mail.ru"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red
               

#define MAX_DAY_VOLATILITY             2000

//   
extern bool      i_useAlert            = false;
extern bool      i_usePush             = false;
extern int       i_indBarsCount        = 5000;

//  
double g_buySignal[];
double g_sellSignal[];
double g_highVolume[];
double g_bullPower[];
double g_bearPower[];


//    
bool g_activate,                                                                                   //    
     g_init;                                                                                       //          ..
                                                                                                   // .. 
int g_volumesArray[MAX_DAY_VOLATILITY];                                                            //     

double g_tickSize;

     
#define DAY_SUNDAY                     0
#define DAY_SATURDAY                   6

//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int init()
{
   g_activate = false;                                                                             //   
   g_init = true;
   
   if (!IsTuningParametersCorrect())                                                               //      -   
      return (-1);                                 
           
   if (!BuffersBind())
      return (-1);                                                   
           
   g_activate = true;                                                                              //   
   return(0);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsTuningParametersCorrect()
{
   string name = WindowExpertName();

   if (Period() > PERIOD_H4)
   {
      Print(name, ":      ,  H4.");
      return (false);
   }

   int period = Period();
   if (period == 0)
   {
      Alert(name, ":    -  0 .  .");
      return (false);
   }
   
   if (Point == 0)
   {
      Alert(name, ":    -    . ",
                  " .");
      return (false);
   }
   
   g_tickSize = MarketInfo(Symbol(), MODE_TICKSIZE);

   return (true);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int deinit()
{
   return(0);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                              |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool BuffersBind()
{
   string name = WindowExpertName();
   IndicatorBuffers(5);

   //     
   if (!SetIndexBuffer(0, g_buySignal)             ||
       !SetIndexBuffer(1, g_sellSignal)            ||
       !SetIndexBuffer(2, g_highVolume)            ||
       !SetIndexBuffer(3, g_bullPower)             ||
       !SetIndexBuffer(4, g_bearPower))
   {
      Alert(name, ":      .  ", GetLastError());
      return (false);
   }

   //    
   for (int i = 0; i < 2; i++)   
      SetIndexStyle(i, DRAW_ARROW);
      
   SetIndexArrow(0, 233);
   SetIndexArrow(1, 234);
      
   SetIndexLabel(0, "Buy signal");
   SetIndexLabel(1, "Sell signal");
   
   return (true);
}
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                     |
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double NP(double value)
{
   if (g_tickSize == 0)
      return (0);
      
   return (MathRound(value / g_tickSize) * g_tickSize);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                     |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void BuffersInitialize()
{
   ArrayInitialize(g_buySignal, EMPTY_VALUE);
   ArrayInitialize(g_sellSignal, EMPTY_VALUE);
   ArrayInitialize(g_highVolume, EMPTY_VALUE);
   ArrayInitialize(g_bullPower, EMPTY_VALUE);
   ArrayInitialize(g_bearPower, EMPTY_VALUE);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|   ,                                                                                                                                 |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int GetRecalcIndex(int& total)
{
   //      
   static int lastBarsCnt;                                                
   if (g_init)                                     
   {                                               
      lastBarsCnt = 0;
      g_init = false;
   }
   total = Bars - 2;                                                                               //    ,       
                                                   
   //          ?
   if (i_indBarsCount > 0 && i_indBarsCount < total)
      total = i_indBarsCount;                      
                                                   
   //       , . .          ,     ,      
   if (lastBarsCnt < Bars - 1)                     
   {                                               
      lastBarsCnt = Bars;
      BuffersInitialize();
      return (total);
   }
   
   //   .            ,    
   int newBarsCnt = Bars - lastBarsCnt;
   lastBarsCnt = Bars;
   return (newBarsCnt);                            
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                   |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double GetMedianValue(int& array[], int cnt)
{
   ArraySort(array, cnt);
   
   //                . 
   //       -      
   int halfElement = cnt / 2;
   if (MathMod(cnt, 2) == 0)
      return ((array[halfElement] + array[halfElement - 1]) / 2);

   return (array[halfElement]);   
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                    |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int GetMedianOfPrevDayVolume(int index)
{
   ArrayInitialize(g_volumesArray, 0);
   int arraySize = 0;
   for (int i = index; i < Bars; i++)
   {
      int dayOfWeek = TimeDayOfWeek(Time[i]);
      if (dayOfWeek == DAY_SUNDAY || dayOfWeek == DAY_SATURDAY)
         continue;
   
      g_volumesArray[arraySize] = Volume[i];
      arraySize++;
      if (arraySize >= MAX_DAY_VOLATILITY)
         break;

      if (TimeDayOfYear(Time[i]) != TimeDayOfYear(Time[i + 1]))
         break;
   }
   
   return (GetMedianValue(g_volumesArray, arraySize));
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                   |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ProcessHighVolume(int index, int dayOfCurBar, int dayOfPrevBar)
{
   //        
   static int maxVolume;
   if (dayOfCurBar != dayOfPrevBar)
      maxVolume = GetMedianOfPrevDayVolume(index + 1);

   //       
   if (Volume[index] <= maxVolume)
   {
      g_highVolume[index] = EMPTY_VALUE;
      return;
   }

   maxVolume = Volume[index];
   g_highVolume[index] = maxVolume;
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double GetTypicalPrice(int index)
{
   return (NP((High[index] + Low[index] + Close[index]) / 3));
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|    ,  ?                                                                                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsFirstMoreThanSecond(double first, double second)
{
   return (first - second > Point / 10);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|    ?                                                                                                                                                                   |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsValuesEquals(double first, double second)
{
   return (MathAbs(first - second) < Point / 10);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                  |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ProcessTotalVolume(int index, int dayOfCurBar, int dayOfPrevBar)
{
   //        
   if (dayOfCurBar != dayOfPrevBar)
   {
      g_bullPower[index] = 0;
      g_bearPower[index] = 0;
      return;
   }

   //       
   double priceCur = GetTypicalPrice(index);
   double pricePrev = GetTypicalPrice(index + 1);
   
   //        -  
   g_bullPower[index] = g_bullPower[index + 1];
   g_bearPower[index] = g_bearPower[index + 1];
   if (IsValuesEquals(priceCur, pricePrev))
      return;
      
   //    -    
   if (IsFirstMoreThanSecond(priceCur, pricePrev))
   {
      g_bullPower[index] = g_bullPower[index + 1] + Volume[index];
      return;
   }

   //    -    
   g_bearPower[index] = g_bearPower[index + 1] + Volume[index];
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|   Alert                                                                                                                                                                            |          
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void SendAlert(string signalType)
{
   static datetime lastAlert = 0;
   if (lastAlert == Time[0])
      return;
      
   lastAlert = Time[0];
   
   string str = WindowExpertName() + " (" + Symbol() + "):  " + signalType + ".";
   if (i_useAlert)
      Alert(str);
      
   if (i_usePush)
      SendNotification(str);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                   |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void GenerateSignals(int index)
{
   //            ,   
   if (g_highVolume[index + 1] == EMPTY_VALUE || IsValuesEquals(g_bullPower[index + 1], g_bearPower[index + 1]))
      return;
      
   //       
   double priceCur = GetTypicalPrice(index + 1);
   double pricePrev = GetTypicalPrice(index + 2);
   
   //        
   if (IsValuesEquals(priceCur, pricePrev))
      return;
      
   //   -       ,    
   if (IsFirstMoreThanSecond(g_bearPower[index + 1], g_bullPower[index + 1]) && IsFirstMoreThanSecond(priceCur, pricePrev))
   {
      g_buySignal[index] = Low[index] - iATR(NULL, 0, 14, index);
      SendAlert("");
   }

   //   -       ,    
   if (IsFirstMoreThanSecond(g_bullPower[index + 1], g_bearPower[index + 1]) && IsFirstMoreThanSecond(pricePrev, priceCur))
   {
      g_sellSignal[index] = High[index] + iATR(NULL, 0, 14, index);
      SendAlert("");
   }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ShowIndicatorData(int limit, int total)
{
   int dayOfCurBar, dayOfPrevBar;

   for (int i = limit; i >= 0; i--)
   {
      dayOfCurBar = TimeDayOfYear(Time[i]);
      dayOfPrevBar = TimeDayOfYear(Time[i + 1]);
      ProcessHighVolume(i, dayOfCurBar, dayOfPrevBar);
      ProcessTotalVolume(i, dayOfCurBar, dayOfPrevBar);
      GenerateSignals(i);
   }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator iteration function                                                                                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int start()
{
   if (!g_activate)                                                                                //     ,     
      return (0);                                 
                                                   
   int total;   
   int limit = GetRecalcIndex(total);                                                              //     ?

   ShowIndicatorData(limit, total);                                                                //   
   
   return(0);
}